using System;
using System.Collections.Generic;
using System.Net;
using System.Text;
using Newtonsoft.Json;
using System.Web;
using Newtonsoft.Json.Linq;

namespace DisqusSharp
{
    public class Disqus
    {
        #region Ctors

        public Disqus()
        {
            UserApiKey = null;
            Encoder = Encoding.ASCII;
        }

        public Disqus(string userKey)
            : this()
        {
            UserApiKey = userKey;
        }

        #endregion

        #region Properties

        public string UserApiKey { get; set; }

        #endregion

        public Encoding Encoder { get; set; }

        private const string POST = "POST";

        public readonly string Version = "1.1";

        public const string URL_GET_MY_KEY = "http://disqus.com/api/get_my_key/";
        public const string URL_GET_USER_NAME = "http://disqus.com/api/get_user_name/";
        private const string URL_GET_FORUM_LIST = "http://disqus.com/api/get_forum_list/";
        private const string URL_GET_FORUM_API_KEY = "http://disqus.com/api/get_forum_api_key/";
        private const string URL_GET_CATEGORIES_LIST = "http://disqus.com/api/get_categories_list/";
        private const string URL_GET_THREAD_LIST = "http://disqus.com/api/get_thread_list/";
        private const string URL_GET_THREAD_POSTS = "http://disqus.com/api/get_thread_posts/";
        private const string URL_GET_THREAD_BY_URL = "http://disqus.com/api/get_thread_by_url/";
        private const string URL_GET_NUM_POSTS = "http://disqus.com/api/get_num_posts/";
        private const string URL_CREATE_POST = "http://disqus.com/api/create_post/";



        #region Methods

        /// <summary>
        /// When logged in to www.disqus.com, retrieves the UserKey,
        /// else returns empty string.
        /// </summary>
        /// <returns>
        /// Returns UserKey if logged in, else returns empty string.
        /// </returns>
        public string GetUserKey()
        {
            WebClient wc = new WebClient();
            string result = wc.DownloadString(new Uri(URL_GET_MY_KEY));

            return result == "Please login before visiting this page."
                       ? string.Empty : result;
        }

        #region get_user_name

        /// <summary>
        /// Validate's API key and get's username
        /// </summary>
        /// <returns></returns>
        public string GetUserName()
        {
            DisqusResponse disqusResponse = Helper.Post(URL_GET_USER_NAME,
                                                        string.Format(
                                                            "api_version={0}&user_api_key={1}",
                                                            Version, UserApiKey),
                                                        Encoder);
            return disqusResponse.Message.ToString();
        }

        #endregion

        #region get_forum_api_key

        /// <summary>
        /// Get a Forum API key for a specific forum
        /// </summary>
        /// <param name="forumId"></param>
        /// <returns>Returns a string which is API key for a given forum.</returns>
        public string GetForumApiKey(int forumId)
        {
            DisqusResponse disqusResponse = Helper.Get(URL_GET_FORUM_API_KEY,
                                                string.Format(
                                                "api_version={0}&user_api_key={1}&forum_id={2}",
                                                Version, UserApiKey, forumId));
            return disqusResponse.Message.ToString();
        }

        #endregion

        #region get_forum_list

        /// <summary>
        /// Get list of websites that user owns or moderates.
        /// </summary>
        /// <returns>List of websites.</returns>
        [Obsolete("GetForumList() deprecated, use GetWebsites() instead.")]
        public List<Forum> GetForumList()
        {
            return GetForums();
        }

        /// <summary>
        /// Get list of websites that user owns or moderates.
        /// </summary>
        /// <returns>List of websites.</returns>
        public List<Forum> GetForums()
        {
            DisqusResponse disqusResponse = Helper.Get(URL_GET_FORUM_LIST,
                                            string.Format(
                                            "api_version={0}&user_api_key={1}",
                                            Version, UserApiKey));

            return JsonConverters.ConvertGetForumList(disqusResponse.Message.ToString());
        }

        #endregion

        #region get_num_posts

        public Dictionary<int, DisqusNumberOfPost> GetNumberOfPosts(int[] threadIds)
        {
            if (threadIds == null)
                throw new ArgumentNullException("threadIds");
            if (threadIds.Length == 0)
                throw new ArgumentOutOfRangeException("threadIds", "threadIds doesn't contain any thread id.");

            StringBuilder data = new StringBuilder();

            for (int i = 0; i < threadIds.Length; i++)
            {
                if (i != 0)
                    data.Append(",");
                data.Append(threadIds[i]);
            }

            DisqusResponse disqusResponse = Helper.Get(URL_GET_NUM_POSTS,
                                                    string.Format(
                                                    "api_version={0}&user_api_key={1}&thread_ids={2}",
                                                    Version, UserApiKey, data));

            Dictionary<int, int[]> tmp = JsonConvert.DeserializeObject<Dictionary<int, int[]>>(disqusResponse.Message.ToString());
            Dictionary<int, DisqusNumberOfPost> rvalue = new Dictionary<int, DisqusNumberOfPost>();

            foreach (KeyValuePair<int, int[]> keyValuePair in tmp)
            {
                rvalue.Add(keyValuePair.Key,
                           new DisqusNumberOfPost { VisibleComments = keyValuePair.Value[0], TotalComments = keyValuePair.Value[1] });
            }
            return rvalue;
        }

        #endregion

        #region get_categories_list

        public List<Category> GetCategories(int forumId)
        {
            DisqusResponse disqusResponse = Helper.Get(URL_GET_CATEGORIES_LIST,
                                            string.Format("api_version={0}&user_api_key={1}&forum_id={2}",
                                            Version, UserApiKey, forumId));
            return JsonConverters.ConvertGetCategoryList(disqusResponse.Message.ToString());
        }

        #endregion

        #region get_thread_list

        /// <summary>
        /// Gets a list of threads on a website.
        /// </summary>
        /// <param name="forumId">Id of the forum.</param>
        /// <param name="limit">Number of entries that should be included in the response.</param>
        /// <param name="start">Starting point for the query</param>
        /// <param name="categoryId">Filter entries by their category</param>
        /// <returns></returns>
        public List<Thread> GetThreads(int forumId, int limit, int start, int? categoryId)
        {
            // todo: assert limit and start
            if (categoryId != null)
                throw new NotImplementedException("Category ID has to be null for now.");

            string data = string.Format(
                "api_version={0}&user_api_key={1}&forum_id={2}&limit={3}&start={4}",
                Version, UserApiKey, forumId, limit, start);

            DisqusResponse response = Helper.Get(URL_GET_THREAD_LIST, data);

            return JsonConverters.ConvertGetThreadList(response.Message.ToString());
        }

        #endregion

        #region get_thread_posts

        public object GetPosts(string forumApiKey, int threadId, int? limit, int? start,
            DisqusPostTypes? filter, DisqusPostTypes? exclude)
        {
            StringBuilder data = new StringBuilder();
            data.AppendFormat("api_version={0}&user_api_key={1}&forum_api_key={2}&thread_id={3}",
                              Version, UserApiKey, forumApiKey, threadId);
            if (limit != null)
                data.AppendFormat("&limit={0}", limit.Value);
            if (start != null)
                data.AppendFormat("&start={0}", start.Value);

            string strFilter = Helper.GenerateQueryString("filter", filter);
            if (!string.IsNullOrEmpty(strFilter))
                data.AppendFormat("&{0}", strFilter);

            string strExclude = Helper.GenerateQueryString("exclude", exclude);
            if (!string.IsNullOrEmpty(strExclude))
                data.AppendFormat("&{0}", strExclude);

            DisqusResponse disqusResponse = Helper.Get(URL_GET_THREAD_POSTS, data.ToString());

            return disqusResponse.Message;
        }

        #endregion

        #region get_thread_by_url

        /// <summary>
        /// Get Thread by url
        /// </summary>
        /// <param name="url"></param>
        /// <param name="forumApiKey"></param>
        /// <param name="partnerApiKey"></param>
        /// <returns>Thread object or null if url not found.</returns>
        public DisqusThreadWithWebsite GetThreadByUrl(string url, string forumApiKey, string partnerApiKey)
        {
            // seems disqus has wrong documentation, forumApiKey is required.

            if (string.IsNullOrEmpty(forumApiKey))
                throw new ArgumentNullException(forumApiKey);

            StringBuilder data = new StringBuilder();
            data.AppendFormat("api_version={0}&user_api_key={1}&url={2}",
                              Version, UserApiKey, HttpUtility.UrlEncode(url));

            data.AppendFormat("&forum_api_key={0}", forumApiKey);
            if (!string.IsNullOrEmpty(partnerApiKey))
                data.AppendFormat("&partner_api_key={0}", partnerApiKey);

            DisqusResponse disqusResponse = Helper.Get(URL_GET_THREAD_BY_URL, data.ToString());

            // message can be null if url entered is not found.
            if (disqusResponse.Message == null)
                return null;

            return JsonConvert.DeserializeObject<DisqusThreadWithWebsite>(disqusResponse.Message.ToString());
        }

        /// <summary>
        /// Get Thread by Url
        /// </summary>
        /// <param name="url"></param>
        /// <returns></returns>
        public DisqusThreadWithWebsite GetThreadByUrl(string url)
        {
            return GetThreadByUrl(url, string.Empty, string.Empty);
        }

        #endregion

        #region create_post

        public object CreatePost(string forumApiKey, int threadId, string message, string authorName, string authorEmail,
                                 DateTime? createdAt, IPAddress ipAddress, string authorUrl, int? parentPost,
                                 DisqusPostTypes? state)
        {

            StringBuilder sb = new StringBuilder();

            sb.AppendFormat("api_version={0}&user_api_key={1}&thread_id={2}&forum_api_key={3}",
                                          Version, UserApiKey, threadId, forumApiKey);
            sb.AppendFormat("&author_name={0}&author_email={1}",
                            authorName, authorEmail);

            if (ipAddress != null)
                sb.AppendFormat("&ip_address={0}", ipAddress);

            if (!string.IsNullOrEmpty(authorUrl))
                sb.AppendFormat("&author_url={0}", authorUrl);

            if (createdAt != null)
                sb.AppendFormat("&created_at={0}", createdAt.Value.ToString("yyyy-MM-ddTHH:mm"));

            // todo parent_post
            // todo state

            sb.AppendFormat("&message={0}", HttpUtility.HtmlEncode(message));

            Helper.Post(URL_CREATE_POST, sb.ToString(), Encoder);

            return null;
        }

        #endregion



        #endregion

        public static string ToString(DisqusPostTypes disqusPostTypes)
        {
            if (disqusPostTypes == DisqusPostTypes.None)
                return string.Empty;

            bool comma = false;
            StringBuilder sb = new StringBuilder();

            if ((disqusPostTypes & DisqusPostTypes.New) == DisqusPostTypes.New)
            {
                sb.Append("new");
                comma = true;
            }

            if ((disqusPostTypes & DisqusPostTypes.Spam) == DisqusPostTypes.Spam)
            {
                sb.AppendFormat("{0}spam", comma ? "," : string.Empty);
                comma = true;
            }

            if ((disqusPostTypes & DisqusPostTypes.Killed) == DisqusPostTypes.Killed)
                sb.AppendFormat("{0}killed", comma ? "," : string.Empty);

            return sb.ToString();
        }
    }
}